# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - ImprovedMoinSearch library

    @copyright: 2010 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.search import searchPages
from MoinMoin.Page import Page
from MoinMoin import wikiutil
import re

heading_regexp = re.compile(r"^(?P<level>=+)(?P<heading>.*?)(?P=level)$", re.UNICODE | re.MULTILINE)
heading_options_regexp = re.compile(r"heading\s*(?P<min>\d+)?(?:\s*-\s*(?P<max>\d+))?", re.UNICODE)

def getSearchResultPages(request, query, **kw):

    """
    Return matching pages using the given 'request' and search 'query'. Optional
    keyword arguments are passed to the underlying search infrastructure.
    """

    results = searchPages(request, query, **kw)
    return results.hits

def getFirstPageHeading(request, page, min_level=1, max_level=1):

    """
    Using the given 'request', return the first heading in the given 'page'
    having a heading level of at least 'min_level' (which is 1 if not specified)
    and at most 'max_level' (which is 1 if not specified).
    """

    full_page = Page(request, page.page_name)
    body = full_page.get_raw_body()

    for match in heading_regexp.finditer(body):
        level = len(match.group("level"))

        if (min_level is None or level >= min_level) and \
            (max_level is None or level <= max_level):

            return match.group("heading")

    return None

def formatResultPages(request, formatter, pages, paging, format, page_from=0):

    """
    Using the given 'request' and 'formatter', return a formatted string showing
    the result 'pages', providing paging controls when 'paging' is set to a true
    value, and providing page details according to the given 'format'.

    If the optional 'pages_from' parameter is set, the result pages from the
    given result (specified within a range from 0 to the length of the 'pages'
    collection) will be shown.
    """

    if format:
        m = heading_options_regexp.search(format)
    else:
        m = None

    if m:
        format = "heading"
        min_level = int_or_none(m.group("min"))
        max_level = int_or_none(m.group("max"))
    else:
        format = "name"

    # Use paging only when there are enough results.

    results_per_page = request.cfg.search_results_per_page
    paging = paging and len(pages) > results_per_page

    if paging:
        pages_to_show = pages[page_from:page_from + results_per_page]
    else:
        pages_to_show = pages

    # Prepare the output.

    output = []
    output.append(formatter.number_list(on=1, start=page_from + 1))

    for page in pages_to_show:
        output.append(formatter.listitem(on=1))

        if format == "heading":
            text = getFirstPageHeading(request, page, min_level, max_level) or page.page_name
        else:
            text = page.page_name

        output.append(formatter.pagelink(on=1, pagename=page.page_name))
        output.append(formatter.text(text))
        output.append(formatter.pagelink(on=0))
        output.append(formatter.listitem(on=0))

    output.append(formatter.number_list(on=0))

    # Show paging navigation.

    if paging:
        output.append(formatPagingNavigation(request, formatter, pages, page_from))

    return "".join(output)

def formatPagingNavigation(request, formatter, pages, page_from=0):

    """
    Using the given 'request' and 'formatter', return a formatted string showing
    the paging navigation for the result 'pages', according to the 'page_from'
    indicator which provides the current position in the result set.
    """

    page = formatter.page
    pagename = page.page_name
    _ = request.getText

    output = []

    results_per_page = request.cfg.search_results_per_page
    number_of_results = len(pages)

    pages_total = number_of_results / results_per_page
    pages_before = page_from / results_per_page
    pages_after = ((number_of_results - page_from) / results_per_page) - 1

    querydict = wikiutil.parseQueryString(request.query_string)

    output.append(formatter.paragraph(on=1))
    output.append(formatter.text(_("Result pages:")))
    output.append(formatter.text(" "))

    n = 0
    while n < pages_before:
        output.append(formatter.pagelink(on=1, pagename=pagename, querystr=getPagingQueryString(querydict, n * results_per_page)))
        output.append(formatter.text(str(n + 1)))
        output.append(formatter.pagelink(on=0))
        output.append(formatter.text(" "))
        n += 1

    output.append(formatter.text(str(n + 1)))
    output.append(formatter.text(" "))
    n += 1

    while n <= pages_total:
        output.append(formatter.pagelink(on=1, pagename=pagename, querystr=getPagingQueryString(querydict, n * results_per_page)))
        output.append(formatter.text(str(n + 1)))
        output.append(formatter.pagelink(on=0))
        output.append(formatter.text(" "))
        n += 1

    output.append(formatter.paragraph(on=0))

    return "".join(output)

def getPagingQueryString(querydict, page_from):
    querydict["from"] = page_from
    return wikiutil.makeQueryString(querydict)

def int_or_none(x):
    if x is None:
        return x
    else:
        return int(x)

# vim: tabstop=4 expandtab shiftwidth=4
